%--------------------------------------------------------------------------------------------------------------
% Enumerates DirectSound devices and finds input/output device IDs using DSDLL interface.
% This function requires 'dsrec.exe' and 'dsdll.dll' files in the pwd (see project web page).
%
% parameters:
%   dsinp_name - input device name string 
%   dsout_name - output device name string
%
% returns:
%   dsinp - list of input device names
%   dsout - list of output device names
%   dsinp_id - input device ID, if 'dsinp_name' not defined, returns 0 (default device)
%   dsout_id - output device ID, if 'dsout_name' not defined, returns 0 (default device)
%
%
% LICENSE
% -------
% Copyright  2016, Stanislav Maslan <s.maslan@seznam.cz>
%
% This script is free software. It comes without any warranty. You can redistribute it
% and/or modify it as you like as long as this license, the author and link to the 
% DSDLL project web page is included. 
%
% Author: Stanislav Maslan
% E-mail: s.maslan@seznam.cz
% www: http://elektronika.kvalitne.cz/SW/dsdll/dsdll_eng.html
% Revision: V1.0, 2016-08-20
% Tested: Octave 3.6.4+
%--------------------------------------------------------------------------------------------------------------
function [dsinp_id,dsout_id,dsinp,dsout] = ds_enum(dsinp_name,dsout_name)

  % enumerate DirectSound devices to file
  [ec,stdout] = system('dsrec.exe -e enum.csv');
  if ec || ~exist('enum.csv','file')
    % failed
    error(stdout(1,:));
  endif
  
  % read the response file
  csv = csv2cell('enum.csv',';');
  unlink('enum.csv');
  R = size(csv,1);
  C = size(csv,2);
  r = 1;
  if R < 1 || C ~= 2 || ~regexp(csv{r,1},'_dsdll_output_')
    error('DirectSound devices enumeration failed!');
  endif
  
  % parse output devices
  r++;
  dsout = {};
  while r <= R && (~ischar(csv{r,1}) || ~regexp(csv{r,1},'_dsdll_input_'))
    dsout{end+1,1} = csv{r,2};
    r++;
  endwhile
  
  % parse input devices
  r++;
  dsinp = {};
  while r <= R
    dsinp{end+1,1} = csv{r,2};
    r++;
  endwhile
  
  % search for input device by name
  if ~isempty(dsinp_name)
    dsinp_id = find(strcmpi(dsinp,dsinp_name),1);
  else
    dsinp_id = 1;
  endif
  if isempty(dsinp_id)
    error('DirectSound input device not found!');
  endif
  dsinp_id--;
  
  % search for output device by name
  if ~isempty(dsout_name)
    dsout_id = find(strcmpi(dsout,dsout_name),1);
  else
    dsout_id = 1;
  endif
  if isempty(dsout_id)
    error('DirectSound output device not found!');
  endif
  dsout_id--;

endfunction