%--------------------------------------------------------------------------------------------------------------
% Plays and records waveforms using DirectSound interface DSDLL.
% This function requires 'dsrec.exe' and 'dsdll.dll' files in the pwd (see project web page).
% If the playback data are defined, it starts playback of the input data either once or in a loop.
% If requested the 'dsrec.exe' will simultanously record the waveform(s) from the input device.
% Transfer of the playback and captured waveform data is made via *.mat files.  
%
% parameters:
%   dsinp_id - input device ID returned by ds_enum(), use 0 for default capture device
%   dsinp_id - output device ID returned by ds_enum(), use 0 for default output device
%   out_data - waveform data for playback, one column per channel, normalized to <-1;+1>
%            - leave empty to disable playback
%   fs - sampling rate (integer) [Hz]
%   rec_chn - number of channels of recorder (1 or 2)
%   rec_delay - delay between capture start and playback start [s]
%   rec_samples - number of samples to capture, no capture if 0 
%   out_loop - non zero value enables looped playback
%   out_volume - output volume in dB (-100 to 0) dB, use 0 by default for maximum volume
%      
% returns:
%   rec_data - captured data, one column per channel, normalized to <-1;+1>
%
%
% LICENSE
% -------
% Copyright  2016, Stanislav Maslan <s.maslan@seznam.cz>
%
% This script is free software. It comes without any warranty. You can redistribute it
% and/or modify it as you like as long as this license, the author and link to the 
% DSDLL project web page is included. 
%
% Author: Stanislav Maslan
% E-mail: s.maslan@seznam.cz
% www: http://elektronika.kvalitne.cz/SW/dsdll/dsdll_eng.html
% Revision: V1.0, 2016-08-20
% Tested: Octave 3.6.4+
%--------------------------------------------------------------------------------------------------------------
function [rec_data] = ds_run(dsinp_id,dsout_id,out_data,fs,rec_chn = 1,rec_delay = 0,rec_samples = 0,out_loop = 0,out_volume = 0)

  % save playback data to temp file
  if numel(out_data)
    save('-v4','_ds_dll_out.mat','out_data');
    ofile = '_ds_dll_out.mat'; 
  else
    ofile = '""';
  endif
  
  % make loop flag
  if out_loop, loop = ' -l'; else loop = ''; endif
  
  % call the DSDLL interface
  ss = sprintf('dsrec.exe -p -c%d -s%d -d%d -o%d %s -i%d %d _ds_dll_capt.mat -v%dp0%s',int32(rec_chn),int32(fs),int32(1000*rec_delay),int32(dsout_id),ofile,int32(dsinp_id),int32(rec_samples),int32(out_volume*100),loop);
  [ec,stdout] = system(ss);
  
  % remove playback temp file
  unlink('_ds_dll_out.mat');
  
  % check for DSDLL error
  if ec
    error(stdout);
  endif
  
  % try to load captured waveform
  if rec_samples
    load('-v4','_ds_dll_capt.mat','data');    
    rec_data = data;
  else
    rec_data = [];
  endif
  
  % remove captured waveform file
  unlink('_ds_dll_capt.mat');

endfunction