%--------------------------------------------------------------------------------------------------------------
% DSDLL demo: Simple coherent sampling digital sampling phase meter
%--------------------------------------------------------------------------------------------------------------
% This simple demo generates two sinewaves at outputs of the given DirectSound device using DSDLL interface.
% Then it captures the waveforms at both soundcard inputs and uses FFT to calculate phase shift and amplitudes.
% This simple demo can measure only the phase of the generated signals, not externally generated signals
% because those are not coherent! But the generated signal can be connected via various passive/active circuits
% so it can be used as a phase shift meter. However it contains no compensation of the interchannel 
% phase shift error so this must be done manually!
%
% The name of the DirectSound devices must be entered in the script! Type in the name as it is
% shown when the script prints out the enumerated devices. Or leave empty so it will choose your 
% system default.
%
% For optimal function set soundcard output/capture mixing rates to the same as in this script i.e. 
% prefereably 48kHz/24bit. Otherwise Windoze will resample the signals.
%
% This demo requires 'dsrec.exe', 'dsdll.dll' files in the pwd (see project web page).
% It also requires 'ds_run.m' and 'ds_enum.m' functions in the pwd.
%
%
% LICENSE
% -------
% Copyright  2016, Stanislav Maslan <s.maslan@seznam.cz>
%
% This script is free software. It comes without any warranty. You can redistribute it
% and/or modify it as you like as long as this license, the author and link to the 
% DSDLL project web page is included. 
%
% Author: Stanislav Maslan
% E-mail: s.maslan@seznam.cz
% www: http://elektronika.kvalitne.cz/SW/dsdll/dsdll_eng.html
% Revision: V1.0, 2016-08-21
% Tested: Octave 3.6.4+
%--------------------------------------------------------------------------------------------------------------
clear all;
close all;
clc;

% set pwd to this script directory otherwise it won't find the 'dsrec.exe' utility
mfld = [fileparts(mfilename('fullpath')) filesep()];
cd(mfld);


% --- general setup ---
% display DFT spectra
disp_spec = 1;

% --- ADC setup ---
% sampling window size [s]
% use longer period for low frequencies or lower standard deviation of the measurement
T = 0.5;
% wait to settle the signals (at least few hundred ms) [s]
% this is essential to prevent transients in the waveform after playback start
% use longer values for low frequencies 
settle = 0.7;

% --- DDS setup: generate two waves with given phase shift ---
% sampling rate [Hz]
fs = 48000;
% desired frequency to synthesize [Hz]
f = 1000;
% interchannel phase shift [deg]
phi = 90.0;
% amplitudes [-]
amp = 0.5;

% --- string names of the input and output DirectSound devices, leave empty to use default ones
dsinp_name = 'Radek (USB Sound Blaster HD)';
dsout_name = 'Reproduktory (USB Sound Blaster HD)';


% --- print some info
printf('Simple coherent sampling phase-meter by Stanislav Maslan\nusing DirectSound DLL interface DSDLL.\n\n');

% --- enumerate DirectSound devices, find desired devices IDs
[dsinp_id,dsout_id,dsinp,dsout] = ds_enum(dsinp_name,dsout_name);

% print list of DirectSound devices
printf('Output devices:\n');
for k = 1:numel(dsout), printf('#%d - %s\n',k - 1,dsout{k}); endfor
printf('\nCapture devices:\n');
for k = 1:numel(dsinp), printf('#%d - %s\n',k - 1,dsinp{k}); endfor
printf('\n\n');



% --- calculate DDS setup so it is coherent with the loop buffer length 
% possible number of periods per the loop buffer  
M = [1 3 5 7]; 
% possible samples count for every M 
N = round(fs./(f./M));
% possible aligned (coherent) frequencies for every M
fm = fs./(N./M);
% select the M for lowest aligned frequency deviation from the desired one
[v,id] = min(abs(fm - f));
% get the aligned (coherent) frequency [Hz]
f = fm(id);
% get the total number of samples
N = N(id);

% calculate samples count for recorder so it is coherent with the DDS wave period
% and so the record is at least 'T' seconds long 
M = ceil((T*fs)/N)*N;

% finally expand samples count to minimum waveform size for DSDLL,
% becuase DS don't like too short playback loops
if N < 16, N *= ceil(16/N); endif



% --- synthesize the sine waveforms
% generate phase vector [rad]
p(:,1) = [0:N-1]*f/fs*2*pi;

% generate the waveforms for DDS
ugen = amp*[sin(p) sin(p + phi/180.0*pi)];


% --- print actual setup
printf('DDS generator: fs = %d Hz, f = %.3f Hz, phi = %.3f deg\n\n',fs,f,phi);


% === MEASUREMENT LOOP ===
printf('press ENTER to stop the measurement...\n\n');
phixs = [];
meas = 1;
while true

  % send it to DSREC and capture waveforms from a recorder
  % (two input channels, delay before capture starts to settle, M samples to capture and loop the output) 
  u = ds_run(dsinp_id,dsout_id,ugen,fs,2,settle,M,1);
  
  % calculate spectra of the captured voltages, loose upper halfs of the spectra (negative frequencies)
  U = fft(u)(2:floor(M/2),:)*2/M;
  
  % build frequency scale [Hz]
  fu(:,1) = [1:size(U,1)]/M*fs;
  
  % DFT bin number with the analyzed frequency (f)
  fid = round(f/(fs/M));
  
  % extract the DFT bin with analyzed frequency (f) 
  Uf = U(fid,:);
  
  % calculate phase shift between the channels [deg]
  phix = arg(Uf(2)/Uf(1))*180/pi;
  
  % display results
  printf('meas = %5d, phix = %.5f deg, U1 = %8.6f, U2 = %.6f\n',meas,phix,abs(Uf(1)),abs(Uf(2)));
    
  % add ne measurement to list
  phixs(end + 1) = phix;
  meas++;
  
  % plot the phase history
  figure(2);
  plot(phixs,'LineWidth',1.5);
  xlabel('measurement [-]');
  ylabel('phi [deg]');
  grid on;
  
  % display the channel spectra
  if disp_spec
    figure(1);
    loglog(fu,abs(U));
    xlabel('f [Hz]');
    ylabel('U [-]');
  endif
  
  if numel(kbhit(1))
    % leave if key pressed...    
    break;
  endif

endwhile

printf('\nTerminated by user...\n');


%figure(2);
%title('Phase measurement example with Creative X-FI HD USB');
%print('dsdll_octave_synphase.png','-S640,300','-F:7','-dpng');
